@@ -21,20 +21,15 @@
 
 /**
  * SECTION:element-alsasrc
- * @short_description: capture audio from an alsa device
  * @see_also: alsasink, alsamixer
  *
- * <refsect2>
- * <para>
  * This element reads data from an audio card using the ALSA API.
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * Record from a sound card using ALSA and encode to Ogg/Vorbis.
- * </para>
- * <programlisting>
+ * |[
  * gst-launch -v alsasrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=alsasrc.ogg
- * </programlisting>
+ * ]| Record from a sound card using ALSA and encode to Ogg/Vorbis.
  * </refsect2>
  *
  * Last reviewed on 2006-03-01 (0.10.4)
@@ -206,15 +201,11 @@ static void
 gst_alsasrc_class_init (GstAlsaSrcClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
-  GstBaseAudioSrcClass *gstbaseaudiosrc_class;
   GstAudioSrcClass *gstaudiosrc_class;
 
   gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasesrc_class = (GstBaseSrcClass *) klass;
-  gstbaseaudiosrc_class = (GstBaseAudioSrcClass *) klass;
   gstaudiosrc_class = (GstAudioSrcClass *) klass;
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_alsasrc_finalize);
@@ -746,6 +737,7 @@ gst_alsasrc_close (GstAudioSrc * asrc)
   GstAlsaSrc *alsa = GST_ALSA_SRC (asrc);
 
   snd_pcm_close (alsa->handle);
+  alsa->handle = NULL;
 
   if (alsa->mixer) {
     gst_alsa_mixer_free (alsa->mixer);
