@@ -25,53 +25,38 @@
 
 /**
  * SECTION:element-gnomevfssrc
- * @short_description: Read from any GnomeVFS-supported location
  * @see_also: #GstFileSrc, #GstGnomeVFSSink
  *
- * <refsect2>
- * <para>
  * This plugin reads data from a local or remote location specified
  * by an URI. This location can be specified using any protocol supported by
  * the GnomeVFS library. Common protocols are 'file', 'http', 'ftp', or 'smb'.
- * </para>
- * <para>
- * In case the element-gnomevfssrc::iradio-mode property is set and the
+ *
+ * In case the #GstGnomeVFSSrc:iradio-mode property is set and the
  * location is a http resource, gnomevfssrc will send special icecast http
  * headers to the server to request additional icecast metainformation. If
  * the server is not an icecast server, it will display the same behaviour
- * as if the element-gnomevfssrc::iradio-mode property was not set. However,
+ * as if the #GstGnomeVFSSrc:iradio-mode property was not set. However,
  * if the server is in fact an icecast server, gnomevfssrc will output
  * data with a media type of application/x-icy, in which case you will
- * need to use the #ICYDemux element as follow-up element to extract
+ * need to use the #GstICYDemux element as follow-up element to extract
  * the icecast meta data and to determine the underlying media type.
- * </para>
- * <para>
- * Example pipeline:
- * <programlisting>
+ *
+ * <refsect2>
+ * <title>Example launch lines</title>
+ * |[
  * gst-launch -v gnomevfssrc location=file:///home/joe/foo.xyz ! fakesink
- * </programlisting>
- * The above pipeline will simply read a local file and do nothing with the
+ * ]| The above pipeline will simply read a local file and do nothing with the
  * data read. Instead of gnomevfssrc, we could just as well have used the
  * filesrc element here.
- * </para>
- * <para>
- * Another example pipeline:
- * <programlisting>
+ * |[
  * gst-launch -v gnomevfssrc location=smb://othercomputer/foo.xyz ! filesink location=/home/joe/foo.xyz
- * </programlisting>
- * The above pipeline will copy a file from a remote host to the local file
+ * ]| The above pipeline will copy a file from a remote host to the local file
  * system using the Samba protocol.
- * </para>
- * <para>
- * Yet another example pipeline:
- * <programlisting>
+ * |[
  * gst-launch -v gnomevfssrc location=http://music.foobar.com/demo.mp3 ! mad ! audioconvert ! audioresample ! alsasink
- * </programlisting>
- * The above pipeline will read and decode and play an mp3 file from a
+ * ]| The above pipeline will read and decode and play an mp3 file from a
  * web server using the http protocol.
- * </para>
  * </refsect2>
- *
  */
 
 
@@ -101,6 +86,8 @@
 #include <string.h>
 
 #include <gst/gst.h>
+#include <gst/tag/tag.h>
+
 /* gnome-vfs.h doesn't include the following header, which we need: */
 #include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 
@@ -112,7 +99,7 @@ GST_ELEMENT_DETAILS ("GnomeVFS Source",
     "Source/File",
     "Read from any GnomeVFS-supported file",
     "Bastien Nocera <hadess@hadess.net>\n"
-    "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
+    "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
 
 static GStaticMutex count_lock = G_STATIC_MUTEX_INIT;
 static gint ref_count = 0;
@@ -154,6 +141,7 @@ static gboolean gst_gnome_vfs_src_check_get_range (GstBaseSrc * src);
 static gboolean gst_gnome_vfs_src_get_size (GstBaseSrc * src, guint64 * size);
 static GstFlowReturn gst_gnome_vfs_src_create (GstBaseSrc * basesrc,
     guint64 offset, guint size, GstBuffer ** buffer);
+static gboolean gst_gnome_vfs_src_query (GstBaseSrc * src, GstQuery * query);
 
 static GstElementClass *parent_class = NULL;
 
@@ -206,11 +194,9 @@ static void
 gst_gnome_vfs_src_class_init (GstGnomeVFSSrcClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
-  gstelement_class = GST_ELEMENT_CLASS (klass);
   gstbasesrc_class = GST_BASE_SRC_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
@@ -226,7 +212,8 @@ gst_gnome_vfs_src_class_init (GstGnomeVFSSrcClass * klass)
       ARG_HANDLE,
       g_param_spec_boxed ("handle",
           "GnomeVFSHandle", "Handle for GnomeVFS",
-          GST_TYPE_GNOME_VFS_HANDLE, G_PARAM_READWRITE));
+          GST_TYPE_GNOME_VFS_HANDLE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* icecast stuff */
   g_object_class_install_property (gobject_class,
@@ -234,25 +221,24 @@ gst_gnome_vfs_src_class_init (GstGnomeVFSSrcClass * klass)
       g_param_spec_boolean ("iradio-mode",
           "iradio-mode",
           "Enable internet radio mode (extraction of shoutcast/icecast metadata)",
-          FALSE, G_PARAM_READWRITE));
+          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class,
       ARG_IRADIO_NAME,
       g_param_spec_string ("iradio-name",
-          "iradio-name", "Name of the stream", NULL, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class,
-      ARG_IRADIO_GENRE,
-      g_param_spec_string ("iradio-genre",
-          "iradio-genre", "Genre of the stream", NULL, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class,
-      ARG_IRADIO_URL,
-      g_param_spec_string ("iradio-url",
-          "iradio-url",
-          "Homepage URL for radio stream", NULL, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class,
-      ARG_IRADIO_TITLE,
-      g_param_spec_string ("iradio-title",
-          "iradio-title",
-          "Name of currently playing song", NULL, G_PARAM_READABLE));
+          "iradio-name", "Name of the stream", NULL,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, ARG_IRADIO_GENRE,
+      g_param_spec_string ("iradio-genre", "iradio-genre",
+          "Genre of the stream", NULL,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, ARG_IRADIO_URL,
+      g_param_spec_string ("iradio-url", "iradio-url",
+          "Homepage URL for radio stream", NULL,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, ARG_IRADIO_TITLE,
+      g_param_spec_string ("iradio-title", "iradio-title",
+          "Name of currently playing song", NULL,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_stop);
@@ -262,6 +248,7 @@ gst_gnome_vfs_src_class_init (GstGnomeVFSSrcClass * klass)
   gstbasesrc_class->check_get_range =
       GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_check_get_range);
   gstbasesrc_class->create = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_create);
+  gstbasesrc_class->query = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_query);
 }
 
 static void
@@ -273,7 +260,6 @@ gst_gnome_vfs_src_init (GstGnomeVFSSrc * gnomevfssrc)
   gnomevfssrc->curoffset = 0;
   gnomevfssrc->seekable = FALSE;
 
-  gnomevfssrc->icy_caps = NULL;
   gnomevfssrc->iradio_mode = FALSE;
   gnomevfssrc->http_callbacks_pushed = FALSE;
   gnomevfssrc->iradio_name = NULL;
@@ -327,11 +313,6 @@ gst_gnome_vfs_src_finalize (GObject * object)
   g_free (src->iradio_title);
   src->iradio_title = NULL;
 
-  if (src->icy_caps) {
-    gst_caps_unref (src->icy_caps);
-    src->icy_caps = NULL;
-  }
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -348,12 +329,7 @@ gst_gnome_vfs_src_uri_get_type (void)
 static gchar **
 gst_gnome_vfs_src_uri_get_protocols (void)
 {
-  static gchar **protocols = NULL;
-
-  if (!protocols)
-    protocols = gst_gnomevfs_get_supported_uris ();
-
-  return protocols;
+  return gst_gnomevfs_get_supported_uris ();
 }
 
 static const gchar *
@@ -481,36 +457,14 @@ gst_gnome_vfs_src_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
-static char *
-unicodify (const char *str, int len, ...)
-{
-  char *ret = NULL, *cset;
-  va_list args;
-  gsize bytes_read, bytes_written;
-
-  if (g_utf8_validate (str, len, NULL))
-    return g_strndup (str, len >= 0 ? len : strlen (str));
-
-  va_start (args, len);
-  while ((cset = va_arg (args, char *)) != NULL)
-  {
-    if (!strcmp (cset, "locale"))
-      ret = g_locale_to_utf8 (str, len, &bytes_read, &bytes_written, NULL);
-    else
-      ret = g_convert (str, len, "UTF-8", cset,
-          &bytes_read, &bytes_written, NULL);
-    if (ret)
-      break;
-  }
-  va_end (args);
-
-  return ret;
-}
-
 static char *
 gst_gnome_vfs_src_unicodify (const char *str)
 {
-  return unicodify (str, -1, "locale", "ISO-8859-1", NULL);
+  const gchar *env_vars[] = { "GST_ICY_TAG_ENCODING",
+    "GST_TAG_ENCODING", NULL
+  };
+
+  return gst_tag_freeform_string_to_utf8 (str, -1, env_vars);
 }
 
 static void
@@ -543,6 +497,13 @@ gst_gnome_vfs_src_received_headers_callback (gconstpointer in,
   if (!src->iradio_mode)
     return;
 
+  GST_DEBUG_OBJECT (src, "receiving internet radio metadata\n");
+
+  /* FIXME: Could we use "Accept-Ranges: bytes"
+   * http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.5
+   * to enable pull-mode?
+   */
+
   for (i = in_args->headers; i; i = i->next) {
     char *data = (char *) i->data;
     char *key = data;
@@ -556,12 +517,19 @@ gst_gnome_vfs_src_received_headers_callback (gconstpointer in,
     if (!strlen (value))
       continue;
 
+    GST_LOG_OBJECT (src, "data %s", data);
+
     /* Icecast stuff */
     if (strncmp (data, "icy-metaint:", 12) == 0) {      /* ugh */
       if (sscanf (data + 12, "%d", &icy_metaint) == 1) {
-        if (icy_metaint > 0)
-          src->icy_caps = gst_caps_new_simple ("application/x-icy",
+        if (icy_metaint > 0) {
+          GstCaps *icy_caps;
+
+          icy_caps = gst_caps_new_simple ("application/x-icy",
               "metadata-interval", G_TYPE_INT, icy_metaint, NULL);
+          gst_pad_set_caps (GST_BASE_SRC_PAD (src), icy_caps);
+          gst_caps_unref (icy_caps);
+        }
       }
       continue;
     }
@@ -635,6 +603,7 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
   GstBuffer *buf;
   GnomeVFSFileSize readbytes;
   guint8 *data;
+  guint todo;
   GstGnomeVFSSrc *src;
 
   src = GST_GNOME_VFS_SRC (basesrc);
@@ -646,7 +615,7 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
   if (G_UNLIKELY (src->curoffset != offset)) {
     GST_DEBUG ("need to seek");
     if (src->seekable) {
-      GST_DEBUG ("seeking to %lld", offset);
+      GST_DEBUG ("seeking to %" G_GUINT64_FORMAT, offset);
       res = gnome_vfs_seek (src->handle, GNOME_VFS_SEEK_START, offset);
       if (res != GNOME_VFS_OK)
         goto seek_failed;
@@ -656,26 +625,36 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
     }
   }
 
-  buf = gst_buffer_new_and_alloc (size);
-
-  if (src->icy_caps)
-    gst_buffer_set_caps (buf, src->icy_caps);
+  buf = gst_buffer_try_new_and_alloc (size);
+  if (G_UNLIKELY (buf == NULL && size == 0)) {
+    GST_ERROR_OBJECT (src, "Failed to allocate %u bytes", size);
+    return GST_FLOW_ERROR;
+  }
 
   data = GST_BUFFER_DATA (buf);
+
+  todo = size;
+  while (todo > 0) {
+    /* this can return less that we ask for */
+    res = gnome_vfs_read (src->handle, data, todo, &readbytes);
+
+    if (G_UNLIKELY (res == GNOME_VFS_ERROR_EOF || (res == GNOME_VFS_OK
+                && readbytes == 0)))
+      goto eos;
+
+    if (G_UNLIKELY (res != GNOME_VFS_OK))
+      goto read_failed;
+
+    if (readbytes < todo) {
+      data = &data[readbytes];
+      todo -= readbytes;
+    } else {
+      todo = 0;
+    }
+    GST_LOG ("  got size %" G_GUINT64_FORMAT, readbytes);
+  }
   GST_BUFFER_OFFSET (buf) = src->curoffset;
-
-  res = gnome_vfs_read (src->handle, data, size, &readbytes);
-
-  if (G_UNLIKELY (res == GNOME_VFS_ERROR_EOF || (res == GNOME_VFS_OK
-              && readbytes == 0)))
-    goto eos;
-
-  GST_BUFFER_SIZE (buf) = readbytes;
-
-  if (G_UNLIKELY (res != GNOME_VFS_OK))
-    goto read_failed;
-
-  src->curoffset += readbytes;
+  src->curoffset += size;
 
   /* we're done, return the buffer */
   *buffer = buf;
@@ -711,6 +690,28 @@ eos:
   }
 }
 
+static gboolean
+gst_gnome_vfs_src_query (GstBaseSrc * basesrc, GstQuery * query)
+{
+  gboolean ret = FALSE;
+  GstGnomeVFSSrc *src = GST_GNOME_VFS_SRC (basesrc);
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_URI:
+      gst_query_set_uri (query, src->uri_name);
+      ret = TRUE;
+      break;
+    default:
+      ret = FALSE;
+      break;
+  }
+
+  if (!ret)
+    ret = GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
+
+  return ret;
+}
+
 static gboolean
 gst_gnome_vfs_src_is_seekable (GstBaseSrc * basesrc)
 {
@@ -745,7 +746,7 @@ gst_gnome_vfs_src_check_get_range (GstBaseSrc * basesrc)
   if (protocol == NULL)
     goto undecided;
 
-  if (strcmp (protocol, "http") == 0) {
+  if (strcmp (protocol, "http") == 0 || strcmp (protocol, "https") == 0) {
     GST_LOG_OBJECT (src, "blacklisted protocol '%s', no random access possible"
         " (URI=%s)", protocol, GST_STR_NULL (src->uri_name));
     return FALSE;
@@ -891,10 +892,5 @@ gst_gnome_vfs_src_stop (GstBaseSrc * basesrc)
   }
   src->curoffset = 0;
 
-  if (src->icy_caps) {
-    gst_caps_unref (src->icy_caps);
-    src->icy_caps = NULL;
-  }
-
   return TRUE;
 }
