@@ -60,9 +60,15 @@ typedef struct
 } FormatInfo;
 
 static const FormatInfo formats[] = {
-  /* container formats with static descriptions */
+  /* container/tag formats with static descriptions */
   {"application/ogg", "Ogg", FLAG_CONTAINER},
   {"application/vnd.rn-realmedia", "Realmedia", FLAG_CONTAINER},
+  {"application/x-id3", N_("ID3 tag"), FLAG_CONTAINER},
+  {"application/x-ape", N_("APE tag"), FLAG_CONTAINER},
+  {"application/x-apetag", N_("APE tag"), FLAG_CONTAINER},
+  {"application/x-icy", N_("ICY internet radio"), FLAG_CONTAINER},
+  {"application/x-3gp", "3GP", FLAG_CONTAINER},
+  {"multipart/x-mixed-replace", "Multipart", FLAG_CONTAINER},
   {"video/x-fli", "FLI/FLC/FLX Animation", FLAG_CONTAINER},
   {"video/x-flv", "Flash", FLAG_CONTAINER},
   {"video/x-matroska", "Matroska", FLAG_CONTAINER},
@@ -70,14 +76,13 @@ static const FormatInfo formats[] = {
   {"video/x-msvideo", "AVI", FLAG_CONTAINER},
   {"video/x-quicktime", "Quicktime", FLAG_CONTAINER},
   {"video/quicktime", "Quicktime", FLAG_CONTAINER},
+  {"video/mj2", "Motion JPEG 2000", FLAG_CONTAINER},
 
   /* audio formats with static descriptions */
   {"audio/x-ac3", "AC-3 (ATSC A/52)", 0},
   {"audio/ac3", "AC-3 (ATSC A/52)", 0},
   {"audio/x-private-ac3", "DVD AC-3 (ATSC A/52)", 0},
   {"audio/x-private1-ac3", "DVD AC-3 (ATSC A/52)", 0},
-  {"audio/x-adpcm", "ADPCM", 0},
-  {"audio/aiff", "Audio Interchange File Format (AIFF)", 0},
   {"audio/x-alaw", "A-Law", 0},
   {"audio/amr", "Adaptive Multi Rate (AMR)", 0},
   {"audio/AMR", "Adaptive Multi Rate (AMR)", 0},
@@ -85,7 +90,7 @@ static const FormatInfo formats[] = {
   {"audio/iLBC-sh", "Internet Low Bitrate Codec (iLBC)", 0},
   {"audio/ms-gsm", "MS GSM", 0},
   {"audio/qcelp", "QCELP", 0},
-  {"audio/x-adpcm", "ADPCM", 0},        /* TODO: different variants */
+  {"audio/aiff", "Audio Interchange File Format (AIFF)", 0},
   {"audio/x-aiff", "Audio Interchange File Format (AIFF)", 0},
   {"audio/x-alac", N_("Apple Lossless Audio (ALAC)"), 0},
   {"audio/x-amr-nb-sh", "Adaptive Multi Rate NarrowBand (AMR-NB)", 0},
@@ -94,6 +99,7 @@ static const FormatInfo formats[] = {
   {"audio/x-cinepak", "Cinepak Audio", 0},
   {"audio/x-dpcm", "DPCM", 0},
   {"audio/x-dts", "DTS", 0},
+  {"audio/x-private1-dts", "DTS", 0},
   {"audio/x-dv", "DV Audio", 0},
   {"audio/x-flac", N_("Free Lossless Audio Codec (FLAC)"), 0},
   {"audio/x-gsm", "GSM", 0},
@@ -106,15 +112,18 @@ static const FormatInfo formats[] = {
   {"audio/x-mod", "Module Music Format (MOD)", 0},
   {"audio/x-mulaw", "Mu-Law", 0},
   {"audio/x-musepack", "Musepack (MPC)", 0},
+  {"audio/x-nellymoser", "Nellymoser Asao", 0},
   {"audio/x-nist", "Sphere NIST", 0},
   {"audio/x-nsf", "Nintendo NSF", 0},
   {"audio/x-paris", "Ensoniq PARIS", 0},
+  {"audio/x-qdm", "QDesign Music (QDM)", 0},
   {"audio/x-qdm2", "QDesign Music (QDM) 2", 0},
   {"audio/x-ralf-mpeg4-generic", "Real Audio Lossless (RALF)", 0},
   {"audio/x-sds", "SDS", 0},
   {"audio/x-shorten", "Shorten Lossless", 0},
   {"audio/x-sid", "Sid", 0},
   {"audio/x-sipro", "Sipro/ACELP.NET Voice", 0},
+  {"audio/x-siren", "Siren", 0},
   {"audio/x-spc", "SNES-SPC700 Sound File Data", 0},
   {"audio/x-speex", "Speex", 0},
   {"audio/x-svx", "Amiga IFF / SVX8 / SV16", 0},
@@ -146,7 +155,7 @@ static const FormatInfo formats[] = {
   {"video/x-dvd-subpicture", "DVD subpicture", 0},
   {"video/x-ffv", N_("FFMpeg v1"), 0},
   {"video/x-flash-screen", "Flash Screen Video", 0},
-  {"video/x-flash-video", "Flash Video", 0},
+  {"video/x-flash-video", "Sorenson Spark Video", 0},
   {"video/x-h261", "H.261", 0},
   {"video/x-huffyuv", "Huffyuv", 0},
   {"video/x-intel-h263", "Intel H.263", 0},
@@ -193,6 +202,9 @@ static const FormatInfo formats[] = {
   {"image/tiff", "TIFF", 0},
   {"image/x-cmu-raster", "CMU Raster Format", 0},
   {"image/x-icon", "ICO", 0},
+  {"image/x-j2c", "JPEG 2000", 0},
+  {"image/x-jpc", "JPEG 2000", 0},
+  {"image/jp2", "JPEG 2000", 0},
   {"image/x-xcf", "XFC", 0},
   {"image/x-pixmap", "XPM", 0},
   {"image/x-xpixmap", "XPM", 0},
@@ -203,10 +215,17 @@ static const FormatInfo formats[] = {
   /* subtitle formats with static descriptions */
   {"application/x-subtitle-sami", N_("Sami subtitle format"), 0},
   {"application/x-subtitle-tmplayer", N_("TMPlayer subtitle format"), 0},
+  {"application/x-kate", "Kate", 0},
+  {"subtitle/x-kate", N_("Kate subtitle format"), 0},
   /* add variant field to typefinder? { "application/x-subtitle", N_("subtitle"), 0}, */
 
+  /* non-audio/video/container formats */
+  {"hdv/aux-v", "HDV AUX-V", 0},
+  {"hdv/aux-a", "HDV AUX-A", 0},
+
   /* formats with dynamic descriptions */
   {"audio/mpeg", NULL, 0},
+  {"audio/x-adpcm", NULL, 0},
   {"audio/x-mace", NULL, 0},
   {"audio/x-pn-realaudio", NULL, 0},
   {"audio/x-raw-int", NULL, 0},
@@ -343,6 +362,8 @@ format_info_get_desc (const FormatInfo * info, const GstCaps * caps)
       ret = "ITU H.264";
     else if (strcmp (variant, "videosoft") == 0)
       ret = "Videosoft H.264";
+    else if (strcmp (variant, "lead") == 0)
+      ret = "Lead H.264";
     else {
       GST_WARNING ("Unknown H264 variant '%s'", variant);
       ret = "H.264";
@@ -463,6 +484,35 @@ format_info_get_desc (const FormatInfo * info, const GstCaps * caps)
       GST_WARNING ("Unexpected acrversion in %" GST_PTR_FORMAT, caps);
       return g_strdup ("ATI VCR");
     }
+  } else if (strcmp (info->type, "audio/x-adpcm") == 0) {
+    const GValue *layout_val;
+
+    layout_val = gst_structure_get_value (s, "layout");
+    if (layout_val != NULL && G_VALUE_HOLDS_STRING (layout_val)) {
+      const gchar *layout;
+
+      if ((layout = g_value_get_string (layout_val))) {
+        gchar *layout_upper, *ret;
+
+        if (strcmp (layout, "swf") == 0)
+          return g_strdup ("Shockwave ADPCM");
+        if (strcmp (layout, "microsoft") == 0)
+          return g_strdup ("Microsoft ADPCM");
+        if (strcmp (layout, "quicktime") == 0)
+          return g_strdup ("Quicktime ADPCM");
+        if (strcmp (layout, "westwood") == 0)
+          return g_strdup ("Westwood ADPCM");
+        if (strcmp (layout, "yamaha") == 0)
+          return g_strdup ("Yamaha ADPCM");
+        /* FIXME: other layouts: sbpro2, sbpro3, sbpro4, ct, g726, ea,
+         * adx, xa, 4xm, smjpeg, dk4, dk3, dvi */
+        layout_upper = g_ascii_strup (layout, -1);
+        ret = g_strdup_printf ("%s ADPCM", layout_upper);
+        g_free (layout_upper);
+        return ret;
+      }
+    }
+    return g_strdup ("ADPCM");
   } else if (strcmp (info->type, "audio/mpeg") == 0) {
     gint ver = 0, layer = 0;
 
