@@ -34,6 +34,9 @@ GST_DEBUG_CATEGORY_STATIC (gst_play_sink_debug);
 
 #define VOLUME_MAX_DOUBLE 10.0
 
+#define DEFAULT_FLAGS             GST_PLAY_FLAG_AUDIO | GST_PLAY_FLAG_VIDEO | GST_PLAY_FLAG_TEXT | \
+                                  GST_PLAY_FLAG_SOFT_VOLUME
+
 #define GST_PLAY_CHAIN(c) ((GstPlayChain *)(c))
 
 /* holds the common data fields for the audio and video pipelines. We keep them
@@ -113,15 +116,15 @@ typedef struct
   GstElement *sink;             /* custom sink to receive subpicture buffers */
 } GstPlaySubpChain;
 
-#define GST_PLAY_SINK_GET_LOCK(playsink) (((GstPlaySink *)playsink)->lock)
-#define GST_PLAY_SINK_LOCK(playsink)     g_mutex_lock (GST_PLAY_SINK_GET_LOCK (playsink))
-#define GST_PLAY_SINK_UNLOCK(playsink)   g_mutex_unlock (GST_PLAY_SINK_GET_LOCK (playsink))
+#define GST_PLAY_SINK_GET_LOCK(playsink) (&((GstPlaySink *)playsink)->lock)
+#define GST_PLAY_SINK_LOCK(playsink)     g_static_rec_mutex_lock (GST_PLAY_SINK_GET_LOCK (playsink))
+#define GST_PLAY_SINK_UNLOCK(playsink)   g_static_rec_mutex_unlock (GST_PLAY_SINK_GET_LOCK (playsink))
 
 struct _GstPlaySink
 {
   GstBin bin;
 
-  GMutex *lock;
+  GStaticRecMutex lock;
 
   gboolean async_pending;
   gboolean need_async_start;
@@ -169,13 +172,51 @@ struct _GstPlaySink
 struct _GstPlaySinkClass
 {
   GstBinClass parent_class;
+
+    gboolean (*reconfigure) (GstPlaySink * playsink);
 };
 
+static GstStaticPadTemplate audiorawtemplate =
+GST_STATIC_PAD_TEMPLATE ("audio_raw_sink",
+    GST_PAD_SINK,
+    GST_PAD_REQUEST,
+    GST_STATIC_CAPS_ANY);
+static GstStaticPadTemplate audiotemplate =
+GST_STATIC_PAD_TEMPLATE ("audio_sink",
+    GST_PAD_SINK,
+    GST_PAD_REQUEST,
+    GST_STATIC_CAPS_ANY);
+static GstStaticPadTemplate videorawtemplate =
+GST_STATIC_PAD_TEMPLATE ("video_raw_sink",
+    GST_PAD_SINK,
+    GST_PAD_REQUEST,
+    GST_STATIC_CAPS_ANY);
+static GstStaticPadTemplate videotemplate =
+GST_STATIC_PAD_TEMPLATE ("video_sink",
+    GST_PAD_SINK,
+    GST_PAD_REQUEST,
+    GST_STATIC_CAPS_ANY);
+static GstStaticPadTemplate texttemplate = GST_STATIC_PAD_TEMPLATE ("text_sink",
+    GST_PAD_SINK,
+    GST_PAD_REQUEST,
+    GST_STATIC_CAPS_ANY);
+static GstStaticPadTemplate subpictemplate =
+GST_STATIC_PAD_TEMPLATE ("subpic_sink",
+    GST_PAD_SINK,
+    GST_PAD_REQUEST,
+    GST_STATIC_CAPS_ANY);
+
+
 
 /* props */
 enum
 {
   PROP_0,
+  PROP_FLAGS,
+  PROP_MUTE,
+  PROP_VOLUME,
+  PROP_FONT_DESC,
+  PROP_VIS_PLUGIN,
   PROP_LAST
 };
 
@@ -189,7 +230,15 @@ static void gst_play_sink_class_init (GstPlaySinkClass * klass);
 static void gst_play_sink_init (GstPlaySink * playsink);
 static void gst_play_sink_dispose (GObject * object);
 static void gst_play_sink_finalize (GObject * object);
+static void gst_play_sink_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * spec);
+static void gst_play_sink_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * spec);
 
+static GstPad *gst_play_sink_request_new_pad (GstElement * element,
+    GstPadTemplate * templ, const gchar * name);
+static void gst_play_sink_release_request_pad (GstElement * element,
+    GstPad * pad);
 static gboolean gst_play_sink_send_event (GstElement * element,
     GstEvent * event);
 static GstStateChangeReturn gst_play_sink_change_state (GstElement * element,
@@ -201,8 +250,8 @@ static void gst_play_sink_handle_message (GstBin * bin, GstMessage * message);
 
 static const GstElementDetails gst_play_sink_details =
 GST_ELEMENT_DETAILS ("Player Sink",
-    "Generic/Bin/Player",
-    "Autoplug and play media from an uri",
+    "Generic/Bin/Sink",
+    "Convenience sink for multiple streams",
     "Wim Taymans <wim.taymans@gmail.com>");
 
 G_DEFINE_TYPE (GstPlaySink, gst_play_sink, GST_TYPE_BIN);
@@ -220,17 +269,77 @@ gst_play_sink_class_init (GstPlaySinkClass * klass)
 
   gobject_klass->dispose = GST_DEBUG_FUNCPTR (gst_play_sink_dispose);
   gobject_klass->finalize = GST_DEBUG_FUNCPTR (gst_play_sink_finalize);
+  gobject_klass->set_property = GST_DEBUG_FUNCPTR (gst_play_sink_set_property);
+  gobject_klass->get_property = GST_DEBUG_FUNCPTR (gst_play_sink_get_property);
 
+
+  /**
+   * GstPlaySink:flags
+   *
+   * Control the behaviour of playsink.
+   */
+  g_object_class_install_property (gobject_klass, PROP_FLAGS,
+      g_param_spec_flags ("flags", "Flags", "Flags to control behaviour",
+          GST_TYPE_PLAY_FLAGS, DEFAULT_FLAGS,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstPlaySink:volume:
+   *
+   * Get or set the current audio stream volume. 1.0 means 100%,
+   * 0.0 means mute. This uses a linear volume scale.
+   *
+   */
+  g_object_class_install_property (gobject_klass, PROP_VOLUME,
+      g_param_spec_double ("volume", "Volume", "The audio volume, 1.0=100%",
+          0.0, VOLUME_MAX_DOUBLE, 1.0,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_klass, PROP_MUTE,
+      g_param_spec_boolean ("mute", "Mute",
+          "Mute the audio channel without changing the volume", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_klass, PROP_FONT_DESC,
+      g_param_spec_string ("subtitle-font-desc",
+          "Subtitle font description",
+          "Pango font description of font "
+          "to be used for subtitle rendering", NULL,
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_klass, PROP_VIS_PLUGIN,
+      g_param_spec_object ("vis-plugin", "Vis plugin",
+          "the visualization element to use (NULL = default)",
+          GST_TYPE_ELEMENT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_signal_new ("reconfigure", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstPlaySinkClass,
+          reconfigure), NULL, NULL, gst_marshal_BOOLEAN__VOID, G_TYPE_BOOLEAN,
+      0, G_TYPE_NONE);
+
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&audiorawtemplate));
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&audiotemplate));
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&videorawtemplate));
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&videotemplate));
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&texttemplate));
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&subpictemplate));
   gst_element_class_set_details (gstelement_klass, &gst_play_sink_details);
 
   gstelement_klass->change_state =
       GST_DEBUG_FUNCPTR (gst_play_sink_change_state);
   gstelement_klass->send_event = GST_DEBUG_FUNCPTR (gst_play_sink_send_event);
+  gstelement_klass->request_new_pad =
+      GST_DEBUG_FUNCPTR (gst_play_sink_request_new_pad);
+  gstelement_klass->release_pad =
+      GST_DEBUG_FUNCPTR (gst_play_sink_release_request_pad);
 
   gstbin_klass->handle_message =
       GST_DEBUG_FUNCPTR (gst_play_sink_handle_message);
 
-  GST_DEBUG_CATEGORY_INIT (gst_play_sink_debug, "playsink", 0, "play bin");
+  klass->reconfigure = GST_DEBUG_FUNCPTR (gst_play_sink_reconfigure);
 }
 
 static void
@@ -243,9 +352,9 @@ gst_play_sink_init (GstPlaySink * playsink)
   playsink->text_sink = NULL;
   playsink->volume = 1.0;
   playsink->font_desc = NULL;
-  playsink->flags = GST_PLAY_FLAG_SOFT_VOLUME;
+  playsink->flags = DEFAULT_FLAGS;
 
-  playsink->lock = g_mutex_new ();
+  g_static_rec_mutex_init (&playsink->lock);
   playsink->need_async_start = TRUE;
   GST_OBJECT_FLAG_SET (playsink, GST_ELEMENT_IS_SINK);
 }
@@ -329,7 +438,7 @@ gst_play_sink_finalize (GObject * object)
 
   playsink = GST_PLAY_SINK (object);
 
-  g_mutex_free (playsink->lock);
+  g_static_rec_mutex_free (&playsink->lock);
 
   G_OBJECT_CLASS (gst_play_sink_parent_class)->finalize (object);
 }
@@ -818,7 +927,7 @@ try_element (GstPlaySink * playsink, GstElement * element, gboolean unref)
 }
 
 /* make the element (bin) that contains the elements needed to perform
- * video display. 
+ * video display.
  *
  *  +------------------------------------------------------------+
  *  | vbin                                                       |
@@ -828,7 +937,7 @@ try_element (GstPlaySink * playsink, GstElement * element, gboolean unref)
  *  |   |  +-------+   +----------+   +----------+   +---------+ |
  * sink-+                                                        |
  *  +------------------------------------------------------------+
- *           
+ *
  */
 static GstPlayVideoChain *
 gen_video_chain (GstPlaySink * playsink, gboolean raw, gboolean async,
@@ -837,7 +946,7 @@ gen_video_chain (GstPlaySink * playsink, gboolean raw, gboolean async,
   GstPlayVideoChain *chain;
   GstBin *bin;
   GstPad *pad;
-  GstElement *head, *prev, *elem;
+  GstElement *head, *prev, *elem = NULL;
 
   chain = g_new0 (GstPlayVideoChain, 1);
   chain->chain.playsink = playsink;
@@ -848,17 +957,21 @@ gen_video_chain (GstPlaySink * playsink, gboolean raw, gboolean async,
   if (playsink->video_sink) {
     GST_DEBUG_OBJECT (playsink, "trying configured videosink");
     chain->sink = try_element (playsink, playsink->video_sink, FALSE);
-  }
-  if (chain->sink == NULL) {
-    GST_DEBUG_OBJECT (playsink, "trying autovideosink");
-    elem = gst_element_factory_make ("autovideosink", "videosink");
-    chain->sink = try_element (playsink, elem, TRUE);
-  }
-  /* FIXME: if DEFAULT_VIDEOSINK != "autovideosink" try this now */
-  if (chain->sink == NULL) {
-    GST_DEBUG_OBJECT (playsink, "trying xvimagesink");
-    elem = gst_element_factory_make ("xvimagesink", "videosink");
-    chain->sink = try_element (playsink, elem, TRUE);
+  } else {
+    /* only try fallback if no specific sink was chosen */
+    if (chain->sink == NULL) {
+      GST_DEBUG_OBJECT (playsink, "trying autovideosink");
+      elem = gst_element_factory_make ("autovideosink", "videosink");
+      chain->sink = try_element (playsink, elem, TRUE);
+    }
+    if (chain->sink == NULL) {
+      /* if default sink from config.h is different then try it too */
+      if (strcmp (DEFAULT_VIDEOSINK, "autovideosink")) {
+        GST_DEBUG_OBJECT (playsink, "trying " DEFAULT_VIDEOSINK);
+        elem = gst_element_factory_make (DEFAULT_VIDEOSINK, "videosink");
+        chain->sink = try_element (playsink, elem, TRUE);
+      }
+    }
   }
   if (chain->sink == NULL)
     goto no_sinks;
@@ -956,13 +1069,24 @@ no_sinks:
   {
     if (!elem) {
       post_missing_element_message (playsink, "autovideosink");
-      GST_ELEMENT_ERROR (playsink, CORE, MISSING_PLUGIN,
-          (_("Both autovideosink and xvimagesink elements are missing.")),
-          (NULL));
+      if (strcmp (DEFAULT_VIDEOSINK, "autovideosink")) {
+        post_missing_element_message (playsink, DEFAULT_VIDEOSINK);
+        GST_ELEMENT_ERROR (playsink, CORE, MISSING_PLUGIN,
+            (_("Both autovideosink and %s elements are missing."),
+                DEFAULT_VIDEOSINK), (NULL));
+      } else {
+        GST_ELEMENT_ERROR (playsink, CORE, MISSING_PLUGIN,
+            (_("The autovideosink element is missing.")), (NULL));
+      }
     } else {
-      GST_ELEMENT_ERROR (playsink, CORE, STATE_CHANGE,
-          (_("Both autovideosink and xvimagesink elements are not working.")),
-          (NULL));
+      if (strcmp (DEFAULT_VIDEOSINK, "autovideosink")) {
+        GST_ELEMENT_ERROR (playsink, CORE, STATE_CHANGE,
+            (_("Both autovideosink and %s elements are not working."),
+                DEFAULT_VIDEOSINK), (NULL));
+      } else {
+        GST_ELEMENT_ERROR (playsink, CORE, MISSING_PLUGIN,
+            (_("The autovideosink element is not working.")), (NULL));
+      }
     }
     free_chain ((GstPlayChain *) chain);
     return NULL;
@@ -1020,7 +1144,7 @@ setup_video_chain (GstPlaySink * playsink, gboolean raw, gboolean async,
  *  |          +-----+      | textoverlay |        |
  *  |          | csp | +--video_sink      |        |
  * sink-------sink  src+ +-text_sink     src--+    |
- *  |          +-----+   |  +-------------+   +-- src   
+ *  |          +-----+   |  +-------------+   +-- src
  * text_sink-------------+                         |
  *  +----------------------------------------------+
  */
@@ -1179,7 +1303,7 @@ gen_text_chain (GstPlaySink * playsink)
  *  |       +-------+    +-----+      |   dvdspu    |        |
  *  |       | queue |    | csp | +---video          |        |
  * sink----sink    src--sink  src+ +-subpicture    src--+    |
- *  |       +-------+    +-----+   |  +-------------+   +-- src   
+ *  |       +-------+    +-----+   |  +-------------+   +-- src
  * subpicture----------------------+                         |
  *  +--------------------------------------------------------+
  */
@@ -1201,7 +1325,7 @@ gen_subp_chain (GstPlaySink * playsink)
   gst_object_ref (bin);
   gst_object_sink (bin);
 
-  videosinkpad = subpsinkpad = srcpad = NULL;
+  subpsinkpad = srcpad = NULL;
 
   /* first try to hook the text pad to the custom sink */
   if (playsink->subp_sink) {
@@ -1302,7 +1426,7 @@ gen_subp_chain (GstPlaySink * playsink)
 }
 
 /* make the chain that contains the elements needed to perform
- * audio playback. 
+ * audio playback.
  *
  * We add a tee as the first element so that we can link the visualisation chain
  * to it when requested.
@@ -1323,7 +1447,7 @@ gen_audio_chain (GstPlaySink * playsink, gboolean raw, gboolean queue)
   GstBin *bin;
   gboolean have_volume;
   GstPad *pad;
-  GstElement *head, *prev, *elem;
+  GstElement *head, *prev, *elem = NULL;
 
   chain = g_new0 (GstPlayAudioChain, 1);
   chain->chain.playsink = playsink;
@@ -1335,17 +1459,21 @@ gen_audio_chain (GstPlaySink * playsink, gboolean raw, gboolean queue)
     GST_DEBUG_OBJECT (playsink, "trying configured audiosink %" GST_PTR_FORMAT,
         playsink->audio_sink);
     chain->sink = try_element (playsink, playsink->audio_sink, FALSE);
-  }
-  if (chain->sink == NULL) {
-    GST_DEBUG_OBJECT (playsink, "trying autoaudiosink");
-    elem = gst_element_factory_make ("autoaudiosink", "audiosink");
-    chain->sink = try_element (playsink, elem, TRUE);
-  }
-  /* FIXME: if DEFAULT_AUDIOSINK != "autoaudiosink" try this now */
-  if (chain->sink == NULL) {
-    GST_DEBUG_OBJECT (playsink, "trying alsasink");
-    elem = gst_element_factory_make ("alsasink", "audiosink");
-    chain->sink = try_element (playsink, elem, TRUE);
+  } else {
+    /* only try fallback if no specific sink was chosen */
+    if (chain->sink == NULL) {
+      GST_DEBUG_OBJECT (playsink, "trying autoaudiosink");
+      elem = gst_element_factory_make ("autoaudiosink", "audiosink");
+      chain->sink = try_element (playsink, elem, TRUE);
+    }
+    if (chain->sink == NULL) {
+      /* if default sink from config.h is different then try it too */
+      if (strcmp (DEFAULT_AUDIOSINK, "autoaudiosink")) {
+        GST_DEBUG_OBJECT (playsink, "trying " DEFAULT_AUDIOSINK);
+        elem = gst_element_factory_make (DEFAULT_AUDIOSINK, "audiosink");
+        chain->sink = try_element (playsink, elem, TRUE);
+      }
+    }
   }
   if (chain->sink == NULL)
     goto no_sinks;
@@ -1500,12 +1628,24 @@ no_sinks:
   {
     if (!elem) {
       post_missing_element_message (playsink, "autoaudiosink");
-      GST_ELEMENT_ERROR (playsink, CORE, MISSING_PLUGIN,
-          (_("Both autoaudiosink and alsasink elements are missing.")), (NULL));
+      if (strcmp (DEFAULT_AUDIOSINK, "autoaudiosink")) {
+        post_missing_element_message (playsink, DEFAULT_AUDIOSINK);
+        GST_ELEMENT_ERROR (playsink, CORE, MISSING_PLUGIN,
+            (_("Both autoaudiosink and %s elements are missing."),
+                DEFAULT_AUDIOSINK), (NULL));
+      } else {
+        GST_ELEMENT_ERROR (playsink, CORE, MISSING_PLUGIN,
+            (_("The autoaudiosink element is missing.")), (NULL));
+      }
     } else {
-      GST_ELEMENT_ERROR (playsink, CORE, STATE_CHANGE,
-          (_("Both autoaudiosink and alsasink elements are not working.")),
-          (NULL));
+      if (strcmp (DEFAULT_AUDIOSINK, "autoaudiosink")) {
+        GST_ELEMENT_ERROR (playsink, CORE, STATE_CHANGE,
+            (_("Both autoaudiosink and %s elements are not working."),
+                DEFAULT_AUDIOSINK), (NULL));
+      } else {
+        GST_ELEMENT_ERROR (playsink, CORE, MISSING_PLUGIN,
+            (_("The autoaudiosink element is not working.")), (NULL));
+      }
     }
     free_chain ((GstPlayChain *) chain);
     return NULL;
@@ -1592,7 +1732,7 @@ setup_audio_chain (GstPlaySink * playsink, gboolean raw, gboolean queue)
  *  |   |  +----------+   +------------+   +----------+   +-------+  |  |
  * sink-+                                                            +-src
  *  +-------------------------------------------------------------------+
- *           
+ *
  */
 static GstPlayVisChain *
 gen_vis_chain (GstPlaySink * playsink)
@@ -1736,7 +1876,8 @@ gst_play_sink_reconfigure (GstPlaySink * playsink)
     need_text = (playsink->text_pad != NULL);
     need_subp = (playsink->subp_pad != NULL);
 
-    /* we can't handle both of them yet */
+    /* we can't handle both of them yet (FIXME: wouldn't it be better to just
+     * pick one and ignore the other then instead of erroring out?) */
     if (need_text && need_subp)
       goto subs_and_text;
   } else if (flags & GST_PLAY_FLAG_VIDEO && playsink->video_pad) {
@@ -1807,7 +1948,7 @@ gst_play_sink_reconfigure (GstPlaySink * playsink)
 
         GST_DEBUG_OBJECT (playsink, "unlinking vis chain");
 
-        /* also had visualisation, release the tee srcpad before we then 
+        /* also had visualisation, release the tee srcpad before we then
          * unlink the video from it */
         if (playsink->audio_tee_vissrc) {
           gst_element_release_request_pad (playsink->audio_tee,
@@ -2030,7 +2171,7 @@ subs_but_no_video:
 subs_and_text:
   {
     GST_ELEMENT_ERROR (playsink, STREAM, FORMAT,
-        (_("Can't play a text subtitles and subpictures.")),
+        (_("Can't display both text subtitles and subpictures.")),
         ("Have text pad and subpicture pad"));
     GST_PLAY_SINK_UNLOCK (playsink);
     return FALSE;
@@ -2175,14 +2316,18 @@ gst_play_sink_request_pad (GstPlaySink * playsink, GstPlaySinkType type)
   gboolean created = FALSE;
   gboolean raw = FALSE;
   gboolean activate = TRUE;
+  const gchar *pad_name = NULL;
 
   GST_DEBUG_OBJECT (playsink, "request pad type %d", type);
 
   GST_PLAY_SINK_LOCK (playsink);
   switch (type) {
     case GST_PLAY_SINK_TYPE_AUDIO_RAW:
+      pad_name = "audio_raw_sink";
       raw = TRUE;
     case GST_PLAY_SINK_TYPE_AUDIO:
+      if (pad_name == NULL)
+        pad_name = "audio_sink";
       if (!playsink->audio_tee) {
         GST_LOG_OBJECT (playsink, "creating tee");
         /* create tee when needed. This element will feed the audio sink chain
@@ -2198,19 +2343,22 @@ gst_play_sink_request_pad (GstPlaySink * playsink, GstPlaySinkType type)
       if (!playsink->audio_pad) {
         GST_LOG_OBJECT (playsink, "ghosting tee sinkpad");
         playsink->audio_pad =
-            gst_ghost_pad_new ("audio_sink", playsink->audio_tee_sink);
+            gst_ghost_pad_new (pad_name, playsink->audio_tee_sink);
         created = TRUE;
       }
       playsink->audio_pad_raw = raw;
       res = playsink->audio_pad;
       break;
     case GST_PLAY_SINK_TYPE_VIDEO_RAW:
+      pad_name = "video_raw_sink";
       raw = TRUE;
     case GST_PLAY_SINK_TYPE_VIDEO:
+      if (pad_name == NULL)
+        pad_name = "video_sink";
       if (!playsink->video_pad) {
         GST_LOG_OBJECT (playsink, "ghosting videosink");
         playsink->video_pad =
-            gst_ghost_pad_new_no_target ("video_sink", GST_PAD_SINK);
+            gst_ghost_pad_new_no_target (pad_name, GST_PAD_SINK);
         created = TRUE;
       }
       playsink->video_pad_raw = raw;
@@ -2265,6 +2413,48 @@ gst_play_sink_request_pad (GstPlaySink * playsink, GstPlaySinkType type)
   return res;
 }
 
+static GstPad *
+gst_play_sink_request_new_pad (GstElement * element, GstPadTemplate * templ,
+    const gchar * name)
+{
+  GstPlaySink *psink;
+  GstPad *pad;
+  GstElementClass *klass;
+  GstPlaySinkType type;
+  const gchar *tplname;
+
+  g_return_val_if_fail (templ != NULL, NULL);
+
+  GST_DEBUG_OBJECT (element, "name:%s", name);
+
+  psink = GST_PLAY_SINK (element);
+  klass = GST_ELEMENT_GET_CLASS (element);
+  tplname = GST_PAD_TEMPLATE_NAME_TEMPLATE (templ);
+
+  /* Figure out the GstPlaySinkType based on the template */
+  if (!strcmp (tplname, "audio_sink"))
+    type = GST_PLAY_SINK_TYPE_AUDIO;
+  else if (!strcmp (tplname, "aduio_raw_sink"))
+    type = GST_PLAY_SINK_TYPE_AUDIO_RAW;
+  else if (!strcmp (tplname, "video_sink"))
+    type = GST_PLAY_SINK_TYPE_VIDEO;
+  else if (!strcmp (tplname, "video_raw_sink"))
+    type = GST_PLAY_SINK_TYPE_VIDEO_RAW;
+  else if (!strcmp (tplname, "text_sink"))
+    type = GST_PLAY_SINK_TYPE_TEXT;
+  else if (!strcmp (tplname, "subpicsink"))
+    type = GST_PLAY_SINK_TYPE_SUBPIC;
+  else
+    goto unknown_template;
+
+  pad = gst_play_sink_request_pad (psink, type);
+  return pad;
+
+unknown_template:
+  GST_WARNING_OBJECT (element, "Unknown pad template");
+  return NULL;
+}
+
 void
 gst_play_sink_release_pad (GstPlaySink * playsink, GstPad * pad)
 {
@@ -2302,6 +2492,14 @@ gst_play_sink_release_pad (GstPlaySink * playsink, GstPad * pad)
   }
 }
 
+static void
+gst_play_sink_release_request_pad (GstElement * element, GstPad * pad)
+{
+  GstPlaySink *psink = GST_PLAY_SINK (element);
+
+  gst_play_sink_release_pad (psink, pad);
+}
+
 static void
 gst_play_sink_handle_message (GstBin * bin, GstMessage * message)
 {
@@ -2315,7 +2513,7 @@ gst_play_sink_handle_message (GstBin * bin, GstMessage * message)
       GstFormat format;
       guint64 amount;
       gdouble rate;
-      gboolean flush, intermediate, res, eos;
+      gboolean flush, intermediate, eos;
       guint64 duration;
 
       GST_INFO_OBJECT (playsink, "Handling step-done message");
@@ -2331,9 +2529,7 @@ gst_play_sink_handle_message (GstBin * bin, GstMessage * message)
               gst_event_new_step (GST_FORMAT_TIME, duration, rate, flush,
               intermediate);
 
-          if (!(res =
-                  gst_element_send_event (playsink->audiochain->chain.bin,
-                      event))) {
+          if (!gst_element_send_event (playsink->audiochain->chain.bin, event)) {
             GST_DEBUG_OBJECT (playsink, "Event failed when sent to audio sink");
           }
         }
@@ -2349,12 +2545,32 @@ gst_play_sink_handle_message (GstBin * bin, GstMessage * message)
 
 /* Send an event to our sinks until one of them works; don't then send to the
  * remaining sinks (unlike GstBin)
+ * Special case: If a subpicture or text sink is set we need to send the event
+ * to them in case it's source is different from the a/v stream's source.
  */
 static gboolean
 gst_play_sink_send_event_to_sink (GstPlaySink * playsink, GstEvent * event)
 {
   gboolean res = TRUE;
 
+  if (playsink->textchain && playsink->textchain->sink) {
+    gst_event_ref (event);
+    if ((res = gst_element_send_event (playsink->textchain->chain.bin, event))) {
+      GST_DEBUG_OBJECT (playsink, "Sent event succesfully to text sink");
+    } else {
+      GST_DEBUG_OBJECT (playsink, "Event failed when sent to text sink");
+    }
+  }
+
+  if (playsink->subpchain && playsink->subpchain->sink) {
+    gst_event_ref (event);
+    if ((res = gst_element_send_event (playsink->subpchain->chain.bin, event))) {
+      GST_DEBUG_OBJECT (playsink, "Sent event succesfully to subpicture sink");
+    } else {
+      GST_DEBUG_OBJECT (playsink, "Event failed when sent to subpicture sink");
+    }
+  }
+
   if (playsink->videochain) {
     gst_event_ref (event);
     if ((res = gst_element_send_event (playsink->videochain->chain.bin, event))) {
@@ -2371,6 +2587,7 @@ gst_play_sink_send_event_to_sink (GstPlaySink * playsink, GstEvent * event)
     }
     GST_DEBUG_OBJECT (playsink, "Event failed when sent to audio sink");
   }
+
 done:
   gst_event_unref (event);
   return res;
@@ -2526,3 +2743,69 @@ activate_failed:
     return GST_STATE_CHANGE_FAILURE;
   }
 }
+
+static void
+gst_play_sink_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * spec)
+{
+  GstPlaySink *playsink = GST_PLAY_SINK (object);
+
+  switch (prop_id) {
+    case PROP_FLAGS:
+      gst_play_sink_set_flags (playsink, g_value_get_flags (value));
+      break;
+    case PROP_VOLUME:
+      gst_play_sink_set_volume (playsink, g_value_get_double (value));
+      break;
+    case PROP_MUTE:
+      gst_play_sink_set_mute (playsink, g_value_get_boolean (value));
+      break;
+    case PROP_FONT_DESC:
+      gst_play_sink_set_font_desc (playsink, g_value_get_string (value));
+      break;
+    case PROP_VIS_PLUGIN:
+      gst_play_sink_set_vis_plugin (playsink, g_value_get_object (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, spec);
+      break;
+  }
+}
+
+static void
+gst_play_sink_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * spec)
+{
+  GstPlaySink *playsink = GST_PLAY_SINK (object);
+
+  switch (prop_id) {
+    case PROP_FLAGS:
+      g_value_set_flags (value, gst_play_sink_get_flags (playsink));
+      break;
+    case PROP_VOLUME:
+      g_value_set_double (value, gst_play_sink_get_volume (playsink));
+      break;
+    case PROP_MUTE:
+      g_value_set_boolean (value, gst_play_sink_get_mute (playsink));
+      break;
+    case PROP_FONT_DESC:
+      g_value_take_string (value, gst_play_sink_get_font_desc (playsink));
+      break;
+    case PROP_VIS_PLUGIN:
+      g_value_take_object (value, gst_play_sink_get_vis_plugin (playsink));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, spec);
+      break;
+  }
+}
+
+
+gboolean
+gst_play_sink_plugin_init (GstPlugin * plugin)
+{
+  GST_DEBUG_CATEGORY_INIT (gst_play_sink_debug, "playsink", 0, "play bin");
+
+  return gst_element_register (plugin, "playsink", GST_RANK_NONE,
+      GST_TYPE_PLAY_SINK);
+}
