@@ -34,6 +34,20 @@
 
 G_BEGIN_DECLS
 
+/**
+ * GstTCPProtocol:
+ * @GST_TCP_PROTOCOL_NONE: Raw data transmission
+ * @GST_TCP_PROTOCOL_GDP: #GstBuffers are wrapped and sent/received using the
+ *      GDP protocol.
+ *
+ * This enum is provided by the tcp/multifd elements to configure the format of
+ * data transmission/reception.
+ *
+ * The GDP protocol wraps data buffers in a header that also carries format
+ * information and timestamps. The None value indicates the data is
+ * sent/received as-is. In that case, format information and timestamping
+ * must be transmitted separately, or implicit in the bytestream itself.
+ */
 typedef enum
 {
   GST_TCP_PROTOCOL_NONE,
@@ -44,14 +58,14 @@ gchar * gst_tcp_host_to_ip (GstElement *element, const gchar *host);
 
 gint gst_tcp_socket_write (int socket, const void *buf, size_t count);
 
-void gst_tcp_socket_close (int *socket);
+void gst_tcp_socket_close (GstPollFD *socket);
 
-GstFlowReturn gst_tcp_read_buffer (GstElement * this, int socket, int cancel_fd, GstBuffer **buf);
+GstFlowReturn gst_tcp_read_buffer (GstElement * this, int socket, GstPoll * fdset, GstBuffer **buf);
 
-GstFlowReturn gst_tcp_gdp_read_buffer (GstElement * this, int socket, int cancel_fd, GstBuffer **buf);
-GstFlowReturn gst_tcp_gdp_read_caps (GstElement * this, int socket, int cancel_fd, GstCaps **caps);
+GstFlowReturn gst_tcp_gdp_read_buffer (GstElement * this, int socket, GstPoll * fdset, GstBuffer **buf);
+GstFlowReturn gst_tcp_gdp_read_caps (GstElement * this, int socket, GstPoll * fdset, GstCaps **caps);
 
-GstEvent * gst_tcp_gdp_read_event (GstElement *elem, int socket, int cancel_fd);
+GstEvent * gst_tcp_gdp_read_event (GstElement *elem, int socket, GstPoll * fdset);
 
 gboolean gst_tcp_gdp_write_buffer (GstElement *elem, int socket, GstBuffer *buffer, gboolean fatal, const gchar *host, int port);
 gboolean gst_tcp_gdp_write_event (GstElement *elem, int socket, GstEvent *event, gboolean fatal, const gchar *host, int port);
