@@ -103,6 +103,11 @@ GST_START_TEST (test_multifdsink_gdp_tag)
   g_main_loop_run (loop);
 
   assert_equals_int (n_tags, 1);
+
+  gst_element_set_state (p1, GST_STATE_NULL);
+  gst_object_unref (p1);
+  gst_element_set_state (p2, GST_STATE_NULL);
+  gst_object_unref (p2);
 }
 
 GST_END_TEST;
@@ -151,6 +156,8 @@ buffer_probe_cb (GstPad * pad, GstBuffer * buffer)
     }
     fail_unless (found, "Did not find incoming IN_CAPS buffer %p on caps",
         buffer);
+
+    gst_caps_unref (caps);
   }
 
   return TRUE;
@@ -202,6 +209,11 @@ GST_START_TEST (test_multifdsink_gdp_vorbisenc)
   g_main_loop_run (loop);
 
   assert_equals_int (n_in_caps, 3);
+
+  gst_element_set_state (p1, GST_STATE_NULL);
+  gst_object_unref (p1);
+  gst_element_set_state (p2, GST_STATE_NULL);
+  gst_object_unref (p2);
 }
 
 GST_END_TEST;
@@ -219,12 +231,7 @@ streamheader_suite (void)
 #ifndef GST_DISABLE_PARSE
   tcase_add_test (tc_chain, test_multifdsink_gdp_tag);
 #ifdef HAVE_VORBIS
-#ifdef HAVE_CPU_PPC64
-  g_print ("\n\n***** skipping test test_multifdsink_gdp_vorbisenc.  May fail "
-      "on PPC64 due to compiler bug. See bug #348114 for details\n\n\n");
-  if (0)                        /* this avoids the 'function not used' warning */
-#endif
-    tcase_add_test (tc_chain, test_multifdsink_gdp_vorbisenc);
+  tcase_add_test (tc_chain, test_multifdsink_gdp_vorbisenc);
 #endif
 #endif
 
